@@ -1,7 +1,13 @@
-#include <linux/module.h>
+#include <linux/config.h>
+
+#if defined (CONFIG_MODVERSIONS) && !defined (MODVERSIONS)
+# define MODVERSIONS
+#endif
 
 #include <linux/kernel.h> /* printk() */
-#include <linux/slab.h> /* kmalloc() */
+#include <linux/module.h>
+
+#include <linux/slab.h> 
 #include <linux/fs.h>     /* everything... */
 #include <linux/errno.h>  /* error codes */
 #include <linux/types.h>  /* size_t */
@@ -9,13 +15,14 @@
 #include <linux/proc_fs.h>
 #include <linux/fcntl.h>        /* O_ACCMODE */
 #include <linux/ioport.h>
+#include <linux/interrupt.h>
 #include <linux/pci.h>
 #include <linux/devfs_fs_kernel.h>
 
 #include <linux/sched.h>
 #include <linux/wait.h>
 
-#include "kernel25compat.h"
+#include "kernel26compat.h"
 
 #include <asm/uaccess.h>
 #include <asm/system.h>   /* cli(), *_flags */
@@ -31,6 +38,9 @@
 #include "i810.h"
 #include "interrupt.h"
 
+int debug=0;
+static int all_devices=0;
+
 int num_devices=0;
 
 static char *sdev_id="svgalib_helper";
@@ -39,11 +49,11 @@ static struct sh_pci_device *sh_pci_devs[MAX_NR_DEVICES];
 
 static int irqs[MAX_NR_DEVICES];
 
-#ifdef CONFIG_DEVFS_FS
+#if (defined CONFIG_DEVFS_FS) && (!defined KERNEL_2_6)
 static devfs_handle_t devfs_handle;
 #endif
 
-static int check_io_range(port,device) {
+static int check_io_range(int port, int device) {
     return 1;
 }
 
@@ -66,35 +76,33 @@ static int get_dev(int pcipos, int minor) {
     if(minor>0) {
         return minor;
     } else {
-        int i;
-        int b, d;
-
-        d=pcipos&0xff;
-        b=(pcipos>>8)&0xff;
-        for(i=1;i<num_devices;i++) {
-            if((b==sh_pci_devs[i]->dev->bus->number) && (d==sh_pci_devs[i]->dev->devfn)) 
-                return i;
-        }
-        return 0;
+		if(pcipos>=num_devices || pcipos<1) return 0;
+		return pcipos;
     }
 }
+
 static volatile int vsync=0;
 static wait_queue_head_t vsync_wait;
 
-static void vsync_interrupt(int irq, void *dev_id, struct pt_regs *regs)
+static irqreturn_t vsync_interrupt(int irq, void *dev_id, struct pt_regs *regs)
 {
     struct sh_pci_device *dev = (struct sh_pci_device *)dev_id;
 
     if((char *)dev==sdev_id) {
-        if(!vga_test_vsync(dev))return;    
+        if(!vga_test_vsync(dev)) {
+			return IRQ_NONE;
+		}
         vga_ack_vsync(dev);
     } else {
-        if(!dev->test_vsync(dev)) return;
+        if(!dev->test_vsync(dev)) {
+			return IRQ_NONE;
+		}
         dev->ack_vsync(dev);
     }
 
     vsync=0;
     wake_up_interruptible(&vsync_wait);
+	return IRQ_HANDLED;
 }
 
 
@@ -103,17 +111,17 @@ static int svgalib_helper_ioctl( struct inode *inode, struct file *filp,
 
     io_t iov;
     pcic_t pciv;
-    int minor = minor(inode->i_rdev);
+    int minor = my_minor(inode->i_rdev);
     struct pci_dev *pdev;
     io_string_t iostr;
-    int i, ret;
+    int i=0, ret;
     u8 pb;
     u16 pw;
     u32 pl;
     void *dev_id;
     unsigned char *outb_str;
 
-    if(_IOC_TYPE(cmd)!=SVGALIB_HELPER_IOC_MAGIC) {
+    if(_IOC_TYPE(cmd)!=SVGAHELPER_MAGIC) {
         return -EINVAL;
     }
     
@@ -121,40 +129,39 @@ static int svgalib_helper_ioctl( struct inode *inode, struct file *filp,
 
     switch(_IOC_NR(cmd)) {
 
-        case _IOC_NR(SVGALIB_HELPER_IOCSREPOUTB):
+        case _IOC_NR(SVGAHELPER_REPOUTB):
             copy_from_user(&iostr,(char *)arg,sizeof(iostr));
             if (iostr.length>4096) return -EINVAL;
-	    if ( (outb_str = kmalloc(iostr.length, GFP_KERNEL )) == NULL ) return -ENOMEM;
-	    copy_from_user(outb_str,iostr.string,iostr.length);
-	    if(check_io_range(iostr.port,minor))
-	        for(i=0; i<iostr.length; i++)
-                    outb(outb_str[i], iostr.port);
-                else ret = -EPERM;
+	    	if ( (outb_str = kmalloc(iostr.length, GFP_KERNEL )) == NULL ) return -ENOMEM;
+	    	copy_from_user(outb_str,iostr.string,iostr.length);
+	    	if(check_io_range(iostr.port,minor)) {
+	        	for(i=0; i<iostr.length; i++) outb(outb_str[i], iostr.port);
+			} else ret = -EPERM;
             kfree (outb_str);
             break;
 
-        case _IOC_NR(SVGALIB_HELPER_IOCSOUTB):
+        case _IOC_NR(SVGAHELPER_OUTB):
            copy_from_user(&iov,(char *)arg,sizeof(iov));
            if(check_io_range(iov.port,minor))
                outb(iov.val,iov.port);
                else ret = -EPERM;
         break;
 
-        case _IOC_NR(SVGALIB_HELPER_IOCSOUTW):
+        case _IOC_NR(SVGAHELPER_OUTW):
            copy_from_user(&iov,(char *)arg,sizeof(iov));
            if(check_io_range(iov.port,minor))
                outw(iov.val,iov.port);
                else ret = -EPERM;
         break;
 
-        case _IOC_NR(SVGALIB_HELPER_IOCSOUTL):
+        case _IOC_NR(SVGAHELPER_OUTL):
            copy_from_user(&iov,(char *)arg,sizeof(iov));
            if(check_io_range(iov.port,minor))
                outl(iov.val,iov.port);
                else ret = -EPERM;
         break;
 
-        case _IOC_NR(SVGALIB_HELPER_IOCGINB):
+        case _IOC_NR(SVGAHELPER_INB):
            copy_from_user(&iov,(char *)arg,sizeof(iov));
            if(check_io_range(iov.port,minor))
                iov.val=inb(iov.port);
@@ -162,7 +169,7 @@ static int svgalib_helper_ioctl( struct inode *inode, struct file *filp,
            copy_to_user((char *)arg,&iov,sizeof(iov));
         break;
 
-        case _IOC_NR(SVGALIB_HELPER_IOCGINW):
+        case _IOC_NR(SVGAHELPER_INW):
            copy_from_user(&iov,(char *)arg,sizeof(iov));
            if(check_io_range(iov.port,minor))
                iov.val=inw(iov.port);
@@ -170,7 +177,7 @@ static int svgalib_helper_ioctl( struct inode *inode, struct file *filp,
            copy_to_user((char *)arg,&iov,sizeof(iov));
         break;
 
-        case _IOC_NR(SVGALIB_HELPER_IOCGINL):
+        case _IOC_NR(SVGAHELPER_INL):
            copy_from_user(&iov,(char *)arg,sizeof(iov));
            if(check_io_range(iov.port,minor))
                iov.val=inl(iov.port);
@@ -178,40 +185,40 @@ static int svgalib_helper_ioctl( struct inode *inode, struct file *filp,
            copy_to_user((char *)arg,&iov,sizeof(iov));
         break;
 
-        case _IOC_NR(SVGALIB_HELPER_IOCSWRITEB):
+        case _IOC_NR(SVGAHELPER_WRITEB):
            copy_from_user(&iov,(char *)arg,sizeof(iov));
            writeb(iov.val,iov.port);
         break;
 
-        case _IOC_NR(SVGALIB_HELPER_IOCSWRITEW):
+        case _IOC_NR(SVGAHELPER_WRITEW):
            copy_from_user(&iov,(char *)arg,sizeof(iov));
            writew(iov.val,iov.port);
         break;
 
-        case _IOC_NR(SVGALIB_HELPER_IOCSWRITEL):
+        case _IOC_NR(SVGAHELPER_WRITEL):
            copy_from_user(&iov,(char *)arg,sizeof(iov));
            writel(iov.val,iov.port);
         break;
 
-        case _IOC_NR(SVGALIB_HELPER_IOCGREADB):
+        case _IOC_NR(SVGAHELPER_READB):
            copy_from_user(&iov,(char *)arg,sizeof(iov));
            iov.val=readb(iov.port);
            copy_to_user((char *)arg,&iov,sizeof(iov));
         break;
 
-        case _IOC_NR(SVGALIB_HELPER_IOCGREADW):
+        case _IOC_NR(SVGAHELPER_READW):
            copy_from_user(&iov,(char *)arg,sizeof(iov));
            iov.val=readw(iov.port);
            copy_to_user((char *)arg,&iov,sizeof(iov));
         break;
 
-        case _IOC_NR(SVGALIB_HELPER_IOCGREADL):
+        case _IOC_NR(SVGAHELPER_READL):
            copy_from_user(&iov,(char *)arg,sizeof(iov));
            iov.val=readl(iov.port);
            copy_to_user((char *)arg,&iov,sizeof(iov));
         break;
 
-        case _IOC_NR(SVGALIB_HELPER_IOCGPCIINB):
+        case _IOC_NR(SVGAHELPER_PCIINB):
             copy_from_user(&pciv,(char *)arg,sizeof(pciv));
             pdev = get_pci_dev(pciv.pcipos, minor);
             if(!pdev) return -EINVAL;
@@ -220,7 +227,7 @@ static int svgalib_helper_ioctl( struct inode *inode, struct file *filp,
             copy_to_user((char *)arg,&pciv,sizeof(pciv));
             break;
 
-        case _IOC_NR(SVGALIB_HELPER_IOCGPCIINW):
+        case _IOC_NR(SVGAHELPER_PCIINW):
             copy_from_user(&pciv,(char *)arg,sizeof(pciv));
             pdev = get_pci_dev(pciv.pcipos, minor);
             if(!pdev) return -EINVAL;
@@ -229,7 +236,7 @@ static int svgalib_helper_ioctl( struct inode *inode, struct file *filp,
             copy_to_user((char *)arg,&pciv,sizeof(pciv));
             break;
 
-        case _IOC_NR(SVGALIB_HELPER_IOCGPCIINL):
+        case _IOC_NR(SVGAHELPER_PCIINL):
             copy_from_user(&pciv,(char *)arg,sizeof(pciv));
             pdev = get_pci_dev(pciv.pcipos, minor);
             if(!pdev) return -EINVAL;
@@ -238,7 +245,7 @@ static int svgalib_helper_ioctl( struct inode *inode, struct file *filp,
             copy_to_user((char *)arg,&pciv,sizeof(pciv));
             break;
 
-        case _IOC_NR(SVGALIB_HELPER_IOCGPCIAPLEN):
+        case _IOC_NR(SVGAHELPER_PCIAPLEN):
             copy_from_user(&pciv,(char *)arg,sizeof(pciv));
             i = get_dev(pciv.pcipos, minor);
             if((i==0) | (pciv.address>5)) return -EINVAL;
@@ -246,7 +253,7 @@ static int svgalib_helper_ioctl( struct inode *inode, struct file *filp,
             copy_to_user((char *)arg,&pciv,sizeof(pciv));
             break;
 
-        case _IOC_NR(SVGALIB_HELPER_IOCSPCIOUTB):
+        case _IOC_NR(SVGAHELPER_PCIOUTB):
             copy_from_user(&pciv,(char *)arg,sizeof(pciv));
             pdev = get_pci_dev(pciv.pcipos, minor);
             if(!pdev) return -EINVAL;
@@ -254,7 +261,7 @@ static int svgalib_helper_ioctl( struct inode *inode, struct file *filp,
             pci_write_config_byte(pdev, pciv.address, pb);
             break;
 
-        case _IOC_NR(SVGALIB_HELPER_IOCSPCIOUTW):
+        case _IOC_NR(SVGAHELPER_PCIOUTW):
             copy_from_user(&pciv,(char *)arg,sizeof(pciv));
             pdev = get_pci_dev(pciv.pcipos, minor);
             if(!pdev) return -EINVAL;
@@ -262,7 +269,7 @@ static int svgalib_helper_ioctl( struct inode *inode, struct file *filp,
             pci_write_config_word(pdev, pciv.address, pw);
             break;
 
-        case _IOC_NR(SVGALIB_HELPER_IOCSPCIOUTL):
+        case _IOC_NR(SVGAHELPER_PCIOUTL):
             copy_from_user(&pciv,(char *)arg,sizeof(pciv));
             pdev = get_pci_dev(pciv.pcipos, minor);
             if(!pdev) return -EINVAL;
@@ -270,21 +277,22 @@ static int svgalib_helper_ioctl( struct inode *inode, struct file *filp,
             pci_write_config_dword(pdev, pciv.address, pl);
             break;
 #ifdef __386__
-        case _IOC_NR(SVGALIB_HELPER_IOCGI810GTT):
+        case _IOC_NR(SVGAHELPER_I810GTT):
             i=i810_make_gtt();
             copy_to_user((char *)arg, &i, sizeof(unsigned int));
             break;
-        case _IOC_NR(SVGALIB_HELPER_IOCGI810GTTE):
+        case _IOC_NR(SVGAHELPER_I810GTTE):
             copy_from_user(&i, (char *)arg, sizeof(unsigned int));
             if(i<I810_SIZE)
                 copy_to_user((char *)arg, &i810_gttes[i], sizeof(unsigned long));
                 else return -EINVAL;
             break;
 #endif
-        case _IOC_NR(SVGALIB_HELPER_IOCWAITRETRACE):
+
+        case _IOC_NR(SVGAHELPER_WAITRETRACE):
 
             /* Workaround for nvidia cards, which are not vga compatible */
-            if(!minor && num_devices==2) minor=1;
+//            if(!minor && num_devices==2) minor=1;
 
             if(minor) {
                 i=sh_pci_devs[minor]->dev->irq;
@@ -322,7 +330,22 @@ static int svgalib_helper_ioctl( struct inode *inode, struct file *filp,
             
             if(vsync) return -ERESTARTSYS;
             break;
-        default: 
+
+#ifdef SVGA_HELPER_VIRTUAL
+        case _IOC_NR(SVGAHELPER_VIRT_TO_PHYS):
+			return dhahelper_virt_to_phys((dhahelper_vmi_t *)arg);
+		case _IOC_NR(SVGAHELPER_VIRT_TO_BUS): 
+			return dhahelper_virt_to_bus((dhahelper_vmi_t *)arg);
+		case _IOC_NR(SVGAHELPER_ALLOC_PA):
+			return dhahelper_alloc_pa((dhahelper_mem_t *)arg);
+		case _IOC_NR(SVGAHELPER_FREE_PA): 
+			return dhahelper_free_pa((dhahelper_mem_t *)arg);
+		case _IOC_NR(SVGAHELPER_LOCK_MEM): 
+			return dhahelper_lock_mem((dhahelper_mem_t *)arg);
+		case _IOC_NR(SVGAHELPER_UNLOCK_MEM): 
+			return dhahelper_unlock_mem((dhahelper_mem_t *)arg);
+		default: 
+#endif
             return -EINVAL;
     }
     return ret;
@@ -331,30 +354,78 @@ static int svgalib_helper_ioctl( struct inode *inode, struct file *filp,
 
 static int svgalib_helper_open( struct inode *inode, struct file * filp) {
 
-   int minor = minor(inode->i_rdev);
+   int minor = my_minor(inode->i_rdev);
    
    if(minor>=num_devices) return -ENODEV;
-   
-   MOD_INC_USE_COUNT;
+
+#ifndef KERNEL_2_6
+	MOD_INC_USE_COUNT;
+#else
+	try_module_get(THIS_MODULE);
+#endif
    
    return 0;
 }
 
 static int svgalib_helper_release( struct inode *inode, struct file *filp) {
-   MOD_DEC_USE_COUNT;
-   
+#ifndef KERNEL_2_6
+	MOD_DEC_USE_COUNT;
+#else
+	module_put(THIS_MODULE);
+#endif
+	
    return 0;
 }
 
-int remap_cache(struct vm_area_struct *vma, unsigned long ofs,
-    		unsigned long start, unsigned long end) {
+int remap_cache(struct vm_area_struct *vma, unsigned long off) {
 
-#if defined(__powerpc__)
-	  pgprot_val(vma->vm_page_prot) |= _PAGE_NO_CACHE|_PAGE_GUARDED;
-#elif defined(__i386__)
-	  if (boot_cpu_data.x86 > 3) pgprot_val(vma->vm_page_prot) |= _PAGE_PCD;
+	vma->vm_pgoff = off >> PAGE_SHIFT;
+	/* This is an IO map - tell maydump to skip this VMA */
+	vma->vm_flags |= VM_IO;
+#if defined(__sparc_v9__)
+	vma->vm_flags |= (VM_SHM | VM_LOCKED);
+	if (my_io_remap_page_range(vma, vma->vm_start, off,
+				vma->vm_end - vma->vm_start, vma->vm_page_prot, 0))
+		return -EAGAIN;
+#else
+#if defined(__mc68000__)
+#if defined(CONFIG_SUN3)
+	pgprot_val(vma->vm_page_prot) |= SUN3_PAGE_NOCACHE;
+#else
+	if (CPU_IS_020_OR_030)
+		pgprot_val(vma->vm_page_prot) |= _PAGE_NOCACHE030;
+	if (CPU_IS_040_OR_060) {
+		pgprot_val(vma->vm_page_prot) &= _CACHEMASK040;
+		/* Use no-cache mode, serialized */
+		pgprot_val(vma->vm_page_prot) |= _PAGE_NOCACHE_S;
+	}
 #endif
-          return my_remap_page_range(vma, start, ofs, end-start, vma->vm_page_prot);
+#elif defined(__powerpc__)
+	pgprot_val(vma->vm_page_prot) |= _PAGE_NO_CACHE|_PAGE_GUARDED;
+#elif defined(__alpha__)
+	/* Caching is off in the I/O space quadrant by design.  */
+#elif defined(__i386__) || defined(__x86_64__)
+	if (boot_cpu_data.x86 > 3)
+		pgprot_val(vma->vm_page_prot) |= _PAGE_PCD;
+#elif defined(__arm__) || defined(__mips__)
+	vma->vm_page_prot = pgprot_noncached(vma->vm_page_prot);
+#elif defined(__sh__)
+	pgprot_val(vma->vm_page_prot) &= ~_PAGE_CACHABLE;
+#elif defined(__hppa__)
+	pgprot_val(vma->vm_page_prot) |= _PAGE_NO_CACHE; 
+#elif defined(__ia64__)
+	vma->vm_page_prot = pgprot_writecombine(vma->vm_page_prot);
+#elif defined(__hppa__)
+	pgprot_val(vma->vm_page_prot) |= _PAGE_NO_CACHE; 
+#else
+#warning What do we have to do here??
+#endif
+	if (my_io_remap_page_range(vma, vma->vm_start, off,
+			     vma->vm_end - vma->vm_start, vma->vm_page_prot))
+		return -EAGAIN;
+#endif /* !__sparc_v9__ */
+	return 0;
+
 }
 
 int check_mem(int card, unsigned long start, unsigned long len) {
@@ -399,7 +470,7 @@ int check_mem(int card, unsigned long start, unsigned long len) {
 static int svgalib_helper_mmap(struct file *filp, struct vm_area_struct *vma) {
    unsigned long start=vma->vm_start;
    unsigned long end=vma->vm_end;
-   unsigned long minor = minor(filp->f_dentry->d_inode->i_rdev);
+   unsigned long minor = my_minor(filp->f_dentry->d_inode->i_rdev);
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,3,0)
    unsigned long ofs=vma->vm_pgoff*PAGE_SIZE;
 #else
@@ -407,7 +478,7 @@ static int svgalib_helper_mmap(struct file *filp, struct vm_area_struct *vma) {
 #endif
 
    if(check_mem(minor, ofs, end-start)) return -EPERM;
-   if(remap_cache(vma, ofs, start, end)) return -EAGAIN;
+   if(remap_cache(vma, ofs)) return -EAGAIN;
    return 0;
 }
 
@@ -432,15 +503,19 @@ int init_module(void)
     int result, i, j;
     struct pci_dev *dev=NULL;
 #ifdef CONFIG_DEVFS_FS
+# ifndef KERNEL_2_6
     devfs_handle_t slave_handle;
+# endif
 #endif
     /*
      * Register your major, and accept a dynamic number
      */
      
     printk(KERN_INFO "svgalib_helper: Initializing, version %s\n", versionstr);
-    result = devfs_register_chrdev(SVGALIB_HELPER_MAJOR, "svgalib_helper", &svgalib_helper_fops);
-    if (result < 0) {
+
+	result = devfs_register_chrdev(SVGALIB_HELPER_MAJOR, "svgalib_helper", &svgalib_helper_fops);
+
+	if (result < 0) {
         printk(KERN_WARNING "svgalib_helper: can't get major %d\n",SVGALIB_HELPER_MAJOR);
         return result;
     }
@@ -448,21 +523,33 @@ int init_module(void)
     if((sh_pci_devs[0]=kmalloc(sizeof(struct sh_pci_device),GFP_KERNEL))==NULL) {
         goto nomem_error;
     }
+	
     memset(sh_pci_devs[0],0,sizeof(struct sh_pci_device));
     num_devices=1;
     for(i=1;i<MAX_NR_DEVICES;i++) sh_pci_devs[i]=NULL;
 
 #ifdef CONFIG_DEVFS_FS
+# ifndef KERNEL_2_6
     devfs_handle = devfs_mk_dir ( NULL, "svga_helper", NULL );
     devfs_register_series( devfs_handle,
                "%u", 8, DEVFS_FL_DEFAULT, SVGALIB_HELPER_MAJOR, 0,
                        S_IFCHR |  S_IRUGO | S_IRWXU, &svgalib_helper_fops, NULL ) ;
     devfs_mk_symlink( NULL, "svga", 0, "svga_helper/0", &slave_handle, NULL );
     devfs_auto_unregister( devfs_handle, slave_handle );
+# else
+	devfs_mk_dir ("svga_helper");
+	for (i = 0; i < 8; i++) {
+		devfs_mk_cdev(MKDEV(SVGALIB_HELPER_MAJOR, i),
+				S_IFCHR |  S_IRUGO | S_IRWXU, "svga_helper/%d", i);
+	}
+	devfs_mk_symlink("svga", "svga_helper/0");
+# endif
 #endif /* devfsd support */
 
     if(pci_present()) {
-        while((dev=pci_find_class(PCI_CLASS_DISPLAY_VGA<<8,dev)) && 
+        while((dev= all_devices ? 
+					pci_find_device(PCI_ANY_ID, PCI_ANY_ID, dev) :
+					pci_find_class(PCI_CLASS_DISPLAY_VGA<<8,dev)) && 
               (num_devices<=MAX_NR_DEVICES)) {
             if((sh_pci_devs[num_devices]=kmalloc(sizeof(struct sh_pci_device),GFP_KERNEL))==NULL) {
                 goto nomem_error;
@@ -512,15 +599,19 @@ int init_module(void)
 
     init_waitqueue_head(&vsync_wait);
 
-    EXPORT_NO_SYMBOLS;
-
+#ifndef KERNEL_2_6
+	EXPORT_NO_SYMBOLS;
+#endif
+	
     return 0; /* succeed */
 
-    nomem_error:
+nomem_error:
     for(i=0;i<MAX_NR_DEVICES;i++) 
-        if(sh_pci_devs[i])kfree(sh_pci_devs[i]);     
-    devfs_unregister_chrdev(SVGALIB_HELPER_MAJOR, "svgalib_helper");
-    return result;
+        if(sh_pci_devs[i])kfree(sh_pci_devs[i]);
+
+	devfs_unregister_chrdev(SVGALIB_HELPER_MAJOR, "svgalib_helper");
+		
+	return result;
 }
 
 void cleanup_module(void)
@@ -532,13 +623,29 @@ void cleanup_module(void)
         }
      
 #ifdef CONFIG_DEVFS_FS
-    devfs_unregister(devfs_handle);
+# ifndef KERNEL_2_6
+	devfs_unregister(devfs_handle);
+# else
+	for (i = 0; i < 8; i++)
+		devfs_remove("svga_helper/%d", i);
+	devfs_remove("svga_helper");
+	devfs_remove("svga");
+# endif	
 #endif
+	
     devfs_unregister_chrdev(SVGALIB_HELPER_MAJOR, "svgalib_helper");
+
 }
 
+MODULE_PARM(debug, "i");
+MODULE_PARM_DESC(debug, "Debug output level.");
+
+MODULE_PARM(all_devices, "i");
+MODULE_PARM_DESC(all_devices, "Give access to all PCI devices, regardless of class.");
+
+
 #ifdef MODULE_LICENSE
-MODULE_LICENSE("GPL and additional rights");
+MODULE_LICENSE("GPL");
 #endif
 
 MODULE_AUTHOR("Matan Ziv-Av <matan@svgalib.org>");
