@@ -1,3 +1,4 @@
+#define DEBUG 1
 /*
 Linux Real Mode Interface - A library of DPMI-like functions for Linux.
 
@@ -517,6 +518,9 @@ em_inb(void)
 	asm volatile ("inb (%w1), %b0"
 	 : "=a" (context.vm.regs.eax)
 	 : "d" (context.vm.regs.edx), "0" (context.vm.regs.eax));
+#ifdef DEBUG
+printf("inb  %04x       %02x\n",context.vm.regs.edx,context.vm.regs.eax);
+#endif
 	}
 
 static void
@@ -525,6 +529,9 @@ em_inw(void)
 	asm volatile ("inw (%w1), %w0"
 	 : "=a" (context.vm.regs.eax)
 	 : "d" (context.vm.regs.edx), "0" (context.vm.regs.eax));
+#ifdef DEBUG
+printf("inw  %04x     %04x\n",context.vm.regs.edx,context.vm.regs.eax);
+#endif
 	}
 
 static void
@@ -533,6 +540,9 @@ em_inl(void)
 	asm volatile ("inl (%w1), %0"
 	 : "=a" (context.vm.regs.eax)
 	 : "d" (context.vm.regs.edx));
+#ifdef DEBUG
+printf("inl  %04x %08x\n",context.vm.regs.edx,context.vm.regs.eax);
+#endif
 	}
 
 static void
@@ -541,6 +551,9 @@ em_outb(void)
 	asm volatile ("outb %b0, (%w1)"
 	 : : "a" (context.vm.regs.eax),
 	 "d" (context.vm.regs.edx));
+#ifdef DEBUG
+printf("outb %04x       %02x\n",context.vm.regs.edx,context.vm.regs.eax);
+#endif
 	}
 
 static void
@@ -549,6 +562,9 @@ em_outw(void)
 	asm volatile ("outw %w0, (%w1)"
 	 : : "a" (context.vm.regs.eax),
 	 "d" (context.vm.regs.edx));
+#ifdef DEBUG
+printf("outw %04x     %02x\n",context.vm.regs.edx,context.vm.regs.eax);
+#endif
 	}
 
 static void
@@ -557,6 +573,9 @@ em_outl(void)
 	asm volatile ("outl %0, (%w1)"
 	 : : "a" (context.vm.regs.eax),
 	 "d" (context.vm.regs.edx));
+#ifdef DEBUG
+printf("outl %04x %08x\n",context.vm.regs.edx,context.vm.regs.eax);
+#endif
 	}
 
 static int
