@@ -8,11 +8,9 @@
 #include <fcntl.h>
 #include <sys/types.h>
 #include <sys/mman.h>
-#include <asm/page.h>
 #include "vga.h"
 #include "libvga.h"
 #include "driver.h"
-#include <linux/pci.h>
 #include "timing.h"
 #include "interface.h"
 #include "vgaregs.h"
