@@ -123,7 +123,7 @@ static int saveregs(unsigned char *regs)
     else
 	r = LOOKUPMODE(ega_modes, lastmode);
     if (r == NULL) {
-	printf("svgalib: egadrv.c/saveregs(): internal error\n");
+	fprintf(stderr,"svgalib: egadrv.c/saveregs(): internal error\n");
 	exit(-1);
     }
     memcpy(regs, r, CRT_C + ATT_C + GRA_C + SEQ_C + MIS_C);
@@ -245,7 +245,7 @@ static int ega_init(int force, int par1, int par2)
 {
 
     if (__svgalib_driver_report)
-	printf("Using EGA driver.\n");
+	fprintf(stderr,"Using EGA driver.\n");
 
     /* Read additional modes from file if available */
     if (ega_modes == NULL) {
