@@ -23,7 +23,9 @@ extern "C"
 #define MOUSE_IMPS2 11
 #define MOUSE_PNP 12
 #define MOUSE_WACOM_GRAPHIRE 13
-#define MOUSE_LAST MOUSE_WACOM_GRAPHIRE
+#define MOUSE_DRMOUSE4DS 14
+#define MOUSE_EXPPS2 15
+#define MOUSE_LAST MOUSE_EXPPS2
 
 #define MOUSE_ORIENTATION_VERTICAL 0
 #define MOUSE_ORIENTATION_HORIZONTAL 1
