@@ -10,7 +10,7 @@ void port_rep_outb(unsigned char* string, int length, int port)
   iostr.string = string;
   iostr.length = length;
   
-  ioctl(__svgalib_mem_fd,SVGALIB_HELPER_IOCSREPOUTB,&iostr); 
+  ioctl(__svgalib_mem_fd,SVGAHELPER_REPOUTB,&iostr); 
 }
 
 void port_out(int value, int port)
@@ -19,7 +19,7 @@ void port_out(int value, int port)
     
     iov.val=value;
     iov.port=port;
-    ioctl(__svgalib_mem_fd,SVGALIB_HELPER_IOCSOUTB,&iov);
+    ioctl(__svgalib_mem_fd,SVGAHELPER_OUTB,&iov);
 }
 
 void port_outw(int value, int port)
@@ -28,7 +28,7 @@ void port_outw(int value, int port)
     
     iov.val=value;
     iov.port=port;
-    ioctl(__svgalib_mem_fd,SVGALIB_HELPER_IOCSOUTW,&iov);
+    ioctl(__svgalib_mem_fd,SVGAHELPER_OUTW,&iov);
 }
 
 void port_outl(int value, int port)
@@ -37,7 +37,7 @@ void port_outl(int value, int port)
     
     iov.val=value;
     iov.port=port;
-    ioctl(__svgalib_mem_fd,SVGALIB_HELPER_IOCSOUTL,&iov);
+    ioctl(__svgalib_mem_fd,SVGAHELPER_OUTL,&iov);
 }
 
 int port_in(int port)
@@ -45,7 +45,7 @@ int port_in(int port)
     io_t iov;
     
     iov.port=port;
-    ioctl(__svgalib_mem_fd,SVGALIB_HELPER_IOCGINB,&iov);
+    ioctl(__svgalib_mem_fd,SVGAHELPER_INB,&iov);
 
     return iov.val;
 }
@@ -55,7 +55,7 @@ int port_inw(int port)
     io_t iov;
     
     iov.port=port;
-    ioctl(__svgalib_mem_fd,SVGALIB_HELPER_IOCGINW,&iov);
+    ioctl(__svgalib_mem_fd,SVGAHELPER_INW,&iov);
 
     return iov.val;
 }
@@ -65,7 +65,7 @@ int port_inl(int port)
     io_t iov;
     
     iov.port=port;
-    ioctl(__svgalib_mem_fd,SVGALIB_HELPER_IOCGINL,&iov);
+    ioctl(__svgalib_mem_fd,SVGAHELPER_INL,&iov);
 
     return iov.val;
 }
