@@ -15,30 +15,9 @@
 
 static inline void read_write(unsigned long off)
 {
-    gr_writeb(gr_readb(off) + 1, off);
+    *(GM+off)=*(GM+off)+1;
 }
 
-static inline int RGB2BGR(int c)
-{
-/* a bswap would do the same as the first 3 but in only ONE! cycle. */
-/* However bswap is not supported by 386 */
-
-    if (MODEFLAGS & RGB_MISORDERED)
-#ifdef NO_ASSEMBLY
-	c = ((c >> 0) & 0xff) << 16 |
-	    ((c >> 8) & 0xff) << 8 |
-	    ((c >> 16) & 0xff) << 0;
-#else
-	asm("rorw  $8, %0\n"	/* 0RGB -> 0RBG */
-	    "rorl $16, %0\n"	/* 0RBG -> BG0R */
-	    "rorw  $8, %0\n"	/* BG0R -> BGR0 */
-	    "shrl  $8, %0\n"	/* 0BGR -> 0BGR */
-      : "=q"(c):"0"(c));
-#endif
-    return c;
-}
-
-
 int vga_drawpixel(int x, int y)
 {
     unsigned long offset;
@@ -48,8 +27,7 @@ int vga_drawpixel(int x, int y)
 	     /* select plane */
              __svgalib_outseq(0x02,1 << (x & 3));
 	     /* write color to pixel */
-	     gr_writeb(COL, y * CI.xbytes + (x >> 2));
-
+             *(GM+y * CI.xbytes + (x >> 2))=COL;
 	return 0;
     }
     switch (__svgalib_cur_info.bytesperpixel) {
@@ -69,49 +47,52 @@ int vga_drawpixel(int x, int y)
 	offset = y * CI.xbytes + x;
 
 	if (__svgalib_modeinfo_linearset & IS_LINEAR ) {
-            *(unsigned char *)(LINEAR_POINTER+offset)=COL;
+            *(LINEAR_POINTER+offset)=COL;
         } else {
 	    /* select segment */
 	    vga_setpage(offset >> 16);
     
 	    /* write color to pixel */
-	    gr_writeb(COL, offset & 0xffff);
+            *(GM+(offset&0xffff))=COL;
         }
 	break;
     case 2:
 	offset = y * CI.xbytes + x * 2;
 	if (__svgalib_modeinfo_linearset & IS_LINEAR ) {
-            *(unsigned short *)(LINEAR_POINTER+offset)=COL;
+            *(uint16_t *)(LINEAR_POINTER+offset)=COL;
         } else {
 	    vga_setpage(offset >> 16);
-            gr_writew(COL, offset & 0xffff);
+            *(uint16_t *)(GM+(offset&0xffff))=COL;
         }
 	break;
     case 3:
-	c = RGB2BGR(COL);
+	c = COL;
 	offset = y * CI.xbytes + x * 3;
 	if (__svgalib_modeinfo_linearset & IS_LINEAR ) {
             /* Only on little endian */
-            *(unsigned char *)(LINEAR_POINTER+offset)=COL&0xff;
-            *(unsigned char *)(LINEAR_POINTER+offset+1)=(COL&0xff00)>>8;
-            *(unsigned char *)(LINEAR_POINTER+offset+2)=(COL&0xff0000)>>16;
+            *(LINEAR_POINTER+offset)=COL&0xff;
+            *(LINEAR_POINTER+offset+1)=(COL&0xff00)>>8;
+            *(LINEAR_POINTER+offset+2)=(COL&0xff0000)>>16;
         } else {
 	    vga_setpage(offset >> 16);
 	    switch (offset & 0xffff) {
 	    case 0xfffe:
-	        gr_writew(c, 0xfffe);
+                *(GM+0xfffe)=COL&0xff;
+		*(GM+0xffff)=(COL&0xff00)>>8;
 	        vga_setpage((offset >> 16) + 1);
-	        gr_writeb(c >> 16, 0);
+		*(GM)=(COL&0xff0000)>>16;
 	        break;
 	    case 0xffff:
-	        gr_writeb(c, 0xffff);
+                *(GM+0xffff)=COL&0xff;
 	        vga_setpage((offset >> 16) + 1);
-	        gr_writew(c >> 8, 0);
+		*(GM)=(COL&0xff00)>>8;
+		*(GM+1)=(COL&0xff0000)>>16;
 	        break;
 	    default:
 	        offset &= 0xffff;
-	        gr_writew(c, offset);
-	        gr_writeb(c >> 16, offset + 2);
+                *(GM+offset)=COL&0xff;
+		*(GM+offset+1)=(COL&0xff00)>>8;
+		*(GM+offset+2)=(COL&0xff0000)>>16;
 	        break;
 	    }
         }
@@ -119,11 +100,10 @@ int vga_drawpixel(int x, int y)
     case 4:
 	offset = y * __svgalib_cur_info.xbytes + x * 4;
 	if (__svgalib_modeinfo_linearset & IS_LINEAR ) {
-            *(unsigned int *)(LINEAR_POINTER+offset)=COL;
+            *(uint32_t *)(LINEAR_POINTER+offset)=COL;
         } else {
 	    vga_setpage(offset >> 16);
-	    gr_writel((MODEFLAGS & RGB_MISORDERED) ? (COL << 8) : COL,
-		      offset & 0xffff);
+            *(uint32_t *)(GM+(offset&0xffff))=COL;
         }
 	break;
     }
@@ -166,34 +146,45 @@ int vga_getpixel(int x, int y)
 	return pix;
     case 1:
 	offset = y * CI.xbytes + x;
-
+	if (__svgalib_modeinfo_linearset & IS_LINEAR ) {
+            return *(LINEAR_POINTER+offset);
+        }
 	/* select segment */
 	vga_setpage(offset >> 16);
 	return gr_readb(offset & 0xffff);
     case 2:
 	offset = y * CI.xbytes + x * 2;
+	if (__svgalib_modeinfo_linearset & IS_LINEAR ) {
+            return *(uint16_t *)(LINEAR_POINTER+offset);
+        }
 	vga_setpage(offset >> 16);
 	return gr_readw(offset & 0xffff);
     case 3:
 	offset = y * CI.xbytes + x * 3;
+	if (__svgalib_modeinfo_linearset & IS_LINEAR ) {
+            return (*(LINEAR_POINTER+offset)) + (*(LINEAR_POINTER+offset+1)<<8) + 
+                   (*(LINEAR_POINTER+offset+2)<<16);
+        }
 	vga_setpage(offset >> 16);
 	switch (offset & 0xffff) {
 	case 0xfffe:
 	    pix = gr_readw(0xfffe);
 	    vga_setpage((offset >> 16) + 1);
-	    return RGB2BGR(pix + (gr_readb(0) << 16));
+	    return pix + (gr_readb(0) << 16);
 	case 0xffff:
 	    pix = gr_readb(0xffff);
 	    vga_setpage((offset >> 16) + 1);
-	    return RGB2BGR(pix + (gr_readw(0) << 8));
+	    return pix + (gr_readw(0) << 8);
 	default:
 	    offset &= 0xffff;
-	    return RGB2BGR(gr_readw(offset)
-			   + (gr_readb(offset + 2) << 26));
+	    return gr_readw(offset) + (gr_readb(offset + 2) << 16);
 	}
 	break; 
     case 4:
 	offset = y * __svgalib_cur_info.xbytes + x * 4;
+	if (__svgalib_modeinfo_linearset & IS_LINEAR ) {
+            return *(uint32_t *)(LINEAR_POINTER+offset);
+        }
 	vga_setpage(offset >> 16);
 	return gr_readl(offset & 0xffff);
     }
